/* ioctl.c
 * Linux CAN-bus device driver.
 * Written by Arnaud Westenberg email:arnaud@wanadoo.nl
 * This software is released under the GPL-License.
 * Version 0.7  6 Aug 2001
 */

#include <linux/autoconf.h>
#if defined (CONFIG_MODVERSIONS) && !defined (MODVERSIONS)
#define MODVERSIONS
#endif

#if defined (MODVERSIONS)
#include <linux/modversions.h>
#endif

#include <linux/fs.h>
#include <linux/version.h>

#include "../include/main.h"
#include "../include/ioctl.h"
#include "../include/i82527.h"

int can_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	int i=0;
	unsigned short channel=0;
	unsigned btr0=0, btr1=0;
	struct msgobj_t *obj;
	struct chip_t *chip = objects_p[MINOR_NR]->hostchip;
	struct canfifo_t *fifo = objects_p[MINOR_NR]->fifo;

	/* Initialize hardware pointers */
	if ( (obj = objects_p[MINOR_NR]) == NULL) {
		CANMSG("Could not assign buffer structure\n");
		return -1;
	}
	if ( (chip = obj->hostchip) == NULL) {
		CANMSG("Device is not correctly configured.\n");
		CANMSG("Please reload the driver.\n");
		return -1;
	}

	switch (cmd) {
		case STAT: {
			for (i=0x0; i<0x100; i++)
				CANMSG("0x%x is 0x%x\n",i,can_read_reg(chip,i));
			break;
		}
		case CMD_START: {
			if (chips_p[arg]->chipspecops->start_chip(chip))
				return -1;
			break;
		}
		case CMD_STOP: {
			if (chips_p[arg]->chipspecops->stop_chip(chip))
				return -1;
			break;
		}
		case CONF_FILTER: {
			if (!strcmp(chip->chip_type,"i82527")) {
			
				unsigned char id1, id0;
				id1 = (unsigned char) (arg << 5);
				id0 = (unsigned char) (arg >> 3);

				DEBUGMSG("configuring ID=%lx in message object:
						 %02x, %02x\n", arg, id0, id1);
				can_write_reg(chip,id1,MSG_OFFSET(obj->object) +
								iMSGID1);
				can_write_reg(chip,id0,MSG_OFFSET(obj->object) +
								iMSGID0);
			}

			/* In- and output buffer re-initialization */
			
			fifo->tx_readp = fifo->buf_tx_entry;
			fifo->tx_writep = fifo->buf_tx_entry;
			fifo->rx_readp = fifo->buf_rx_entry;
			fifo->rx_writep = fifo->buf_rx_entry;
			fifo->rx_size= MAX_BUF_LENGTH * sizeof(struct canmsg_t);
			fifo->tx_size = fifo->rx_size;

			#if (LINUX_VERSION_CODE <= KERNEL_VERSION(2,2,19))
				init_waitqueue(&fifo->readq);
				init_waitqueue(&fifo->writeq);
			#else
				init_waitqueue_head(&fifo->readq);
				init_waitqueue_head(&fifo->writeq);
			#endif

			fifo->rx_in_progress = 0;
			fifo->tx_in_progress = 0;

			fifo->head = fifo->tail = 0; //TEMP!!
			
			break;
		}

		case CONF_BAUD: {
			channel = arg & 0xff;
			btr0 = (arg >> 8) & 0xff;
			btr1 = (arg >> 16) & 0xff;

			if (chips_p[channel]->chipspecops->set_btregs(chip,
								btr0, btr1)) {
				CANMSG("Error setting bit timing registers\n");
				return -1;
			}
			break;
		}
		default: {
			CANMSG("Not a valid ioctl command\n");
			return -EINVAL;
		}
		
	}

	return 0;
}
